function [min_position] = APCA_MDL_car(data, dim)


% Input: 1. a time series in a row vector
%        2. a number as a fixed dimensionality
% Output:  the correct cardinality of this data

data = zscore(data);  
car = 2:256;                            

Int_data = DNorm_Unif(data,256);            % data in discrete space

Bits = zeros(1,length(car));                % Bits is the array to store bit cost for each cardinality

model_cost=zeros(1,length(car));
Line_apca = ts2apca(Int_data,dim);          % APCA for data in discrete space

for i=1:length(car);
    
    Int_apca = round(ReduceCard(Line_apca, car(i)));                                   % A line in corresponding reconstruction line in discrete space
    figure(1);
    plot(Int_data);
    hold on;
    plot(Int_apca,'r'); 
    hold off;
    title(sprintf('APCA number of cardinality =%d ', car(i)));
    legend('data','APCA approximation','Location','Best');
    axis([-inf inf -127 128]);
    Bits(i)= calculate_bit_cost(Int_data,Int_apca);    % the dimensionality * bit cost   
    model_cost(i)= log2(car(i))*dim+ceil(log2(length(data)))*(dim-1);                            % this is the function that Eamonn gave to calculate the model cost
    
end

total_cost=Bits+model_cost;

[x y] = min(total_cost); 
min_position  = y+1;
 fprintf('the correct cardinality is at %d', min_position);
 
figure();   

subplot(4,1,1)
plot(car,Bits);
title('APCA MDL: D(T-H)');
xlabel('cardinality');
subplot(4,1,2)
plot(car,model_cost);
title('APCA MDL: model cost'); 
subplot(4,1,3);
plot(car,total_cost); 
title('Total bits cost= model cost + D(T-H)');
subplot(4,1,4);
plot(data);


end

function [Bits]=calculate_bit_cost(Int_data,Int_line_position)
% Input: 1.the integer data,
%        2.reconstruction line poine
% Output: 
%    total_bits :the bit cost of this time series

Int_dist=Int_data-Int_line_position;                              % calculate the distance between Int data and Int reconstruction line;

uniques=unique(Int_dist);             % unique numbers in Int_dist;
numUnique=histc(Int_dist,uniques);    %  how many times each unique number appears

Prob= numUnique/length(Int_dist);

if uniques == 0
    avg = 1;
    Bits = avg*length(Int_data);
else
    
[dict,avg]=huffmandict(uniques,Prob);
Bits=avg*length(Int_data);
end

end


function TS = ts2apca(data,num_segment)
% Convert segment from ACPA to discrete time series
% ======================================
% == Input ==  a time series in a row vector; 
%              number of segments;
%===============================================
% segment : an output from apca()
%       - segment(i).lx  -> left position on x axis
%       - segment(i).rx  -> right position on x axis
%       - segment(i).mc  -> error (don't sure)
%       - segment(i).y   -> y position of this interval
% == Output ==
% TS     : a discrete ts which has the same length as input ts (in high dimensional space) before using apca()
% ======================================
% last updated: Feb 26  2011
   segment=apca1(data,num_segment);
    TS(segment(end).rx)=0;
    for i=1:length(segment)
       TS(segment(i).lx:segment(i).rx)=round(segment(i).y); 
    end

end

function NewTS = ReduceCard(TS,card,f)


% Reduce an inpute TS to discrete space with only "card" different value.
% Divide all space into card+1 equal parts
% ======================================
% == Input ==
% TS   : time series (can be continous ts or discrete ts)
% card : cardinality, number of different symbol in each position in ts
% == Output ==
% TS   : a ts which has the same length as input ts, with only different "card" symbol
% ======================================
    MIN = -127;         %minimum value in TS
    MAX = 128;
    RANGE = MAX-MIN;
        
    CutPoints = MIN:RANGE/(card):MAX;
    Centers = (CutPoints(2:end)+CutPoints(1:end-1))/2;
    CutPoints(end)=[];
    CutPoints(1)=[];
    
    NewTS = zeros(size(TS));
    for i=1:length(TS)
        index = bsearch(TS(i), CutPoints);
        NewTS(i) = Centers(index);
    end
       if (exist('f','var'))
        if (f>0), 
            figure(f);
        else
            figure(); 
        end
        plot(TS,'b','LineWidth',2);
        hold on;
        plot(NewTS,'r','LineWidth',2);
        for i=1:length(Centers)
           if (i<=length(CutPoints))
             plot(1:2000,ones(1,2000)*CutPoints(i),'g:');
           end
           plot(1:2000,ones(1,2000)*Centers(i),'m--');       
        end
        legend('APCA','Approximation','CutPoints','Centers');
        hold off;
        title(sprintf('Card = %d, largest number given to huffman coding is the distance between the black and the red ',card));
       end
      
       
end


function ind = bsearch(a,X)
% Binary search of value a on X
% X is sorted ascending (small to large)
% a is a number
% Output: the first ind which X(ind)>=a
%  ind=end+1 is out of range (larger than all value in X)
    ind = -1;
    st=1;
    en=length(X);
    
    if (X(st)>=a),  ind=st; end
    if (X(en)<=a),  ind=en+1;  end 
    
    while (ind<0)&&(st<en)
        mid=floor((st+en)/2);
        %disp(sprintf('st:%d mid:%d en:%d  X(%d)=%d',st,mid,en,mid,X(mid)));
        if (X(mid)<a)&&(a<=X(mid+1))
            ind = mid+1;
        end
        if (X(mid)<a)
            st=mid+1;
        else
            en=mid;
        end        
    end
end

function DZ = DNorm_Unif(X, card)

% Input: X ts is a subsequence, a row vector
% Output: DZ is a discrete subsequence in range [1,card]

    if (~exist('card','var')), card=256; end;
      
    mn = min(X);
    mx = max(X);
    DZ = round((X-mn)/(mx-mn)*(card-1)) + 1;
    DZ=DZ-card/2;
    %DZ = zscore(DZ);
end
function segment = apca1(data,num_segments,dummy)

data=data';
if num_segments==1
    segment(1).lx=1;
    segment(1).rx=length(data);
    segment(1).y=0;
    segment(1).mc=0;
    
else
    
left_x = [1 : 2 : size(data,1)-1];
right_x      = left_x + 1;
right_x(end) = size(data,1);
number_of_segments = length(left_x );

for i = 1 : number_of_segments 
   
   segment(i).lx = left_x(i);
   segment(i).rx = right_x(i);
   segment(i).mc = inf;
   segment(i).y = inf;
end;

for i = 1 : number_of_segments - 1
   coef = polyfit([segment(i).lx :segment(i+1).rx]',data(segment(i).lx :segment(i+1).rx),0);
   best = (0*( [segment(i).lx :segment(i+1).rx]' ))+coef(1);
   segment(i).mc = sum((data([segment(i).lx :segment(i+1).rx]')-best).^2);
end;

while  length(segment) > num_segments  
      
   [value, i ] = min([segment(:).mc]);
      
     if i > 1 && i < length(segment) -1								
      
    	coef = polyfit([segment(i).lx :segment(i+2).rx]',data(segment(i).lx :segment(i+2).rx),0);        
    	best = (0 *( [segment(i).lx :segment(i+2).rx]' ))+coef(1);       
        segment(i).mc = sum((data([segment(i).lx :segment(i+2).rx]')-best).^2);
	 	segment(i).rx = segment(i+1).rx;
    	segment(i+1) = [];
       	i = i - 1;    
        coef = polyfit([segment(i).lx :segment(i+1).rx]',data(segment(i).lx :segment(i+1).rx),0);
      	best = (0*( [segment(i).lx :segment(i+1).rx]' ))+coef(1);    
        segment(i).mc = sum((data([segment(i).lx :segment(i+1).rx]')-best).^2);
       
   elseif i == 1
       
	   coef = polyfit([segment(i).lx :segment(i+2).rx]',data(segment(i).lx :segment(i+2).rx),0);
       best = (0*( [segment(i).lx :segment(i+2).rx]' ))+coef(1);
       segment(i).mc = sum((data([segment(i).lx :segment(i+2).rx]')-best).^2);
	   segment(i).rx = segment(i+1).rx;
      segment(i+1) = [];
              
   else
     
      segment(i).rx = segment(i+1).rx;
      segment(i).mc = inf;
      segment(i+1) = [];    
      i = i - 1;       
      coef = polyfit([segment(i).lx :segment(i+1).rx]',data(segment(i).lx :segment(i+1).rx),0);
      best = (0*( [segment(i).lx :segment(i+1).rx]' ))+coef(1);    
     segment(i).mc = sum((data([segment(i).lx :segment(i+1).rx]')-best).^2);
  end; 
          
end;


for i = 1 : length(segment) 
   
      coef = polyfit([segment(i).lx :segment(i).rx]',data(segment(i).lx :segment(i).rx),0);
      best = (0*( [segment(i).lx :segment(i).rx]' ))+coef(1);
      segment(i).y = best(1);
end;

residuals=[];
for i = 1 : length(segment)    
      coef = polyfit([segment(i).lx :segment(i).rx]',data(segment(i).lx :segment(i).rx),0);     
      best = (0*( [segment(i).lx :segment(i).rx]' ))+coef(1);
      residuals = [    residuals ; sum((data([segment(i).lx :segment(i).rx]')-best).^2)];
end;

if nargin == 3

hold on;
plot(data+0,'r');
temp = [];
for i = 1 : length(segment) 
   
      coef = polyfit([segment(i).lx :segment(i).rx]',data(segment(i).lx :segment(i).rx),0);
      best = (0*( [segment(i).lx :segment(i).rx]' ))+coef(1);
      plot([segment(i).lx :segment(i).rx]', best,'b');
      temp = [temp; [best(1) best(end)]];    
end;

for i = 1 : length(segment)  - 1 
        plot([segment(i).rx :segment(i+1).lx]', [ temp(i,2) temp(i+1,1)  ],'g');
end;
end

end

end












